from yarl import URL


# comparison and hashing


def test_ne_str():
    url = URL("http://example.com/")
    assert url != "http://example.com/"


def test_eq():
    url = URL("http://example.com/")
    assert url == URL("http://example.com/")


def test_hash():
    assert hash(URL("http://example.com/")) == hash(URL("http://example.com/"))


def test_hash_double_call():
    url = URL("http://example.com/")
    assert hash(url) == hash(url)


def test_le_less():
    url1 = URL("http://example1.com/")
    url2 = URL("http://example2.com/")

    assert url1 <= url2


def test_le_eq():
    url1 = URL("http://example.com/")
    url2 = URL("http://example.com/")

    assert url1 <= url2


def test_le_not_implemented():
    url = URL("http://example1.com/")

    assert url.__le__(123) is NotImplemented


def test_lt():
    url1 = URL("http://example1.com/")
    url2 = URL("http://example2.com/")

    assert url1 < url2


def test_lt_not_implemented():
    url = URL("http://example1.com/")

    assert url.__lt__(123) is NotImplemented


def test_ge_more():
    url1 = URL("http://example1.com/")
    url2 = URL("http://example2.com/")

    assert url2 >= url1


def test_ge_eq():
    url1 = URL("http://example.com/")
    url2 = URL("http://example.com/")

    assert url2 >= url1


def test_ge_not_implemented():
    url = URL("http://example1.com/")

    assert url.__ge__(123) is NotImplemented


def test_gt():
    url1 = URL("http://example1.com/")
    url2 = URL("http://example2.com/")

    assert url2 > url1


def test_gt_not_implemented():
    url = URL("http://example1.com/")

    assert url.__gt__(123) is NotImplemented



from typing import overload, Any, Tuple, Optional, Mapping, Union, Sequence, Type
import multidict

QueryVariable = Union[str, int]
Query = Union[
    None, str, Mapping[str, QueryVariable], Sequence[Tuple[str, QueryVariable]]
]

class URL:
    scheme: str
    raw_user: str
    user: Optional[str]
    raw_password: Optional[str]
    password: Optional[str]
    raw_host: Optional[str]
    host: Optional[str]
    port: Optional[int]
    raw_path: str
    path: str
    raw_query_string: str
    query_string: str
    path_qs: str
    raw_path_qs: str
    raw_fragment: str
    fragment: str
    query: multidict.MultiDict[str]
    raw_name: str
    name: str
    raw_parts: Tuple[str, ...]
    parts: Tuple[str, ...]
    parent: URL
    def __init__(
        self, val: Union[str, "URL"] = ..., *, encoded: bool = ...
    ) -> None: ...
    @classmethod
    def build(
        cls,
        *,
        scheme: str = ...,
        user: Optional[str] = ...,
        password: Optional[str] = ...,
        host: str = ...,
        port: Optional[int] = ...,
        path: str = ...,
        query: Optional[Query] = ...,
        query_string: str = ...,
        fragment: str = ...,
        encoded: bool = ...
    ) -> URL: ...
    def __str__(self) -> str: ...
    def __repr__(self) -> str: ...
    def __eq__(self, other: Any) -> bool: ...
    def __le__(self, other: Any) -> bool: ...
    def __lt__(self, other: Any) -> bool: ...
    def __ge__(self, other: Any) -> bool: ...
    def __gt__(self, other: Any) -> bool: ...
    def __hash__(self) -> int: ...
    def __truediv__(self, name: str) -> URL: ...
    def is_absolute(self) -> bool: ...
    def is_default_port(self) -> bool: ...
    def origin(self) -> URL: ...
    def relative(self) -> URL: ...
    def with_scheme(self, scheme: str) -> URL: ...
    def with_user(self, user: Optional[str]) -> URL: ...
    def with_password(self, password: Optional[str]) -> URL: ...
    def with_host(self, host: str) -> URL: ...
    def with_port(self, port: Optional[int]) -> URL: ...
    def with_path(self, path: str, *, encoded: bool = ...) -> URL: ...
    @overload
    def with_query(self, query: Query) -> URL: ...
    @overload  # noqa: F811
    def with_query(self, **kwargs: QueryVariable) -> URL: ...
    @overload
    def update_query(self, query: Query) -> URL: ...
    @overload  # noqa: F811
    def update_query(self, **kwargs: QueryVariable) -> URL: ...
    def with_fragment(self, fragment: Optional[str]) -> URL: ...
    def with_name(self, name: str) -> URL: ...
    def join(self, url: URL) -> URL: ...
    def human_repr(self) -> str: ...
    # private API
    @classmethod
    def _normalize_path(cls, path: str) -> str: ...

class cached_property:
    def __init__(self, wrapped: Any) -> None: ...
    def __get__(self, inst: URL, owner: Type[URL]) -> Any: ...
    def __set__(self, inst: URL, value: Any) -> None: ...



import pathlib
import os
import sys
import re

from setuptools import setup, Extension


if sys.version_info < (3, 6):
    raise RuntimeError("yarl 1.4+ requires Python 3.6+")


NO_EXTENSIONS = bool(os.environ.get("YARL_NO_EXTENSIONS"))  # type: bool

if sys.implementation.name != "cpython":
    NO_EXTENSIONS = True


extensions = [Extension("yarl._quoting", ["yarl/_quoting.c"])]
# extra_compile_args=["-g"],
# extra_link_args=["-g"],


here = pathlib.Path(__file__).parent
fname = here / "yarl" / "__init__.py"

with fname.open(encoding="utf8") as fp:
    try:
        version = re.findall(r'^__version__ = "([^"]+)"$', fp.read(), re.M)[0]
    except IndexError:
        raise RuntimeError("Unable to determine version.")

install_requires = ["multidict>=4.0", "idna>=2.0"]


def read(name):
    fname = here / name
    with fname.open(encoding="utf8") as f:
        return f.read()


args = dict(
    name="yarl",
    version=version,
    description=("Yet another URL library"),
    long_description="\n\n".join([read("README.rst"), read("CHANGES.rst")]),
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Internet :: WWW/HTTP",
    ],
    author="Andrew Svetlov",
    author_email="andrew.svetlov@gmail.com",
    url="https://github.com/aio-libs/yarl/",
    license="Apache 2",
    packages=["yarl"],
    install_requires=install_requires,
    python_requires=">=3.6",
    include_package_data=True,
)


if not NO_EXTENSIONS:
    print("**********************")
    print("* Accellerated build *")
    print("**********************")
    setup(ext_modules=extensions, **args)
else:
    print("*********************")
    print("* Pure Python build *")
    print("*********************")
    setup(**args)
